from __future__ import generators
import plus
import AI
from AI import vector3
import Arenas
import Gooey
import math
import Tactics

class ENTEngage(AI.Tactic):
    name = "ENTEngage"
    
    def Evaluate(self):
        self.priority = 0
        self.target_id, range = self.ai.GetNearestEnemy()
        
        # TODO: come up with a priority scale for "engaging"
        
        if self.target_id != None:
            self.priority = 80
        else:
            self.priority -= 100
                
    def Execute(self):
        if self.target_id != None:
            self.ai.enemy_id = self.target_id
            
            # default turning & throttle off
            self.ai.Throttle(0)
            self.ai.Turn(0)
            
            target = plus.getLocation(self.target_id)
            
            # slow down if we're coming in too fast
            if self.ai.RobotInRange(self.target_id)[0] and self.ai.GetSpeed() > self.ai.top_speed / 2:
                self.ai.Throttle(-self.ai.max_throttle)
                
            # try to get within range of chassis...
            if not self.ai.RobotInRange(self.target_id)[1]:                
                self.ai.DriveToLocation(target)
                
            else:               
                # ... then aim for center of chassis
                h = self.ai.GetHeadingTo(target, False)
                self.ai.AimToHeading(h)
                    
            return True
        else:
            return False

class UpsideDownTracker:
    def __init__(self):
        self.last_time = 0
        self.last_position = None
        self.contact = 0
        self.stoptime = 0
        
class ENTITY(AI.SuperAI):
    "Flipper strategy"
    name = "ENTITY"

    def __init__(self, **args):
        AI.SuperAI.__init__(self, **args)
               
        self.flipZone = "Flip" 
        self.pushZone = "Push" 
        self.flameZone = "Flame"
        
        self.FlipIn = "Flip"        
        self.PushIn = "Push"
        self.FlameIn = "Flame"        
         
        self.move_thresh = 0
        self.flippedBias = 0                    
        self.NoChassisTime = 0
        self.move_time = 0  
        self.cooldown = 0     
        self.sritime = 8
        self.reloadTime = 0
        self.reloadDelay = 0
        self.flipDelay = 0
        self.flipLimit = 1000
        self.StuckTimer = 0
        self.SrimechMove = 0
        
        self.upTrack = {}      # for seeing if enemies can drive upside down  
        self.push = 0 
        self.botinzone = 0
        self.flame = 0
     
        
        self.flipComp = []
        self.sriComp = []  
        
        if 'SriMove' in args: 
            self.SrimechMove = args['SriMove']  

        if 'EnemyMoveRadius' in args: 
            self.move_thresh = args['EnemyMoveRadius']  
        
        if 'FlipBias' in args: 
            self.flippedBias = args['FlipBias']                        
        
        if 'NoChassisTime' in args: 
            self.NoChassisTime = args.get('NoChassisTime') * 4        

        if 'EnemyMoveTime' in args: 
            self.move_time = args['EnemyMoveTime'] 

        if 'EnemyFlipCooldown' in args: 
            self.cooldown = args['EnemyFlipCooldown'] 
        
        if 'SrimechInterval' in args: 
            self.sritime = args.get('SrimechInterval') * 8
         
        if 'ReloadTime' in args: 
            self.reloadDelay = args.get('ReloadTime') * 8
            
        if 'FlipDelay' in args:
            self.flipDelay = args.get('FlipDelay')
            
        if 'FlipLimit' in args:
            self.flipLimit = args['FlipLimit']        
            
        if 'flipComp' in args: 
            self.flipComp = list(args['flipComp'])
            
        if 'sriComp' in args: 
            self.sriComp = list(args['sriComp'])   
            
        if 'tactic' in args: 
            self.theTactic = args['tactic']
            
            if   self.theTactic  == "Charge":
                self.tactics.append(Tactics.Charge(self))
                
            elif self.theTactic  == "Ram":
                self.tactics.append(Tactics.Ram(self))
                
            elif self.theTactic  == "Shove":
                self.tactics.append(Tactics.Shove(self))
                
            elif self.theTactic  == "Engage":
                self.tactics.append(Tactics.Engage(self))
                
            else:
                self.tactics.append(ENTEngage(self))
        else: 
            self.tactics.append(ENTEngage(self))
        
        
        
    def Activate(self, active):
        # Activate flame control at the start of the match, leaving it alone in def Tick
        self.Input("Flame", 0, 100)        
        if active:
            if AI.SuperAI.debugging:
                self.debug = Gooey.Plain("watch", 0, 75, 100, 75)
                tbox = self.debug.addText("line0", 0, 0, 100, 15)
                tbox.setText("Throttle")
                tbox = self.debug.addText("line1", 0, 15, 100, 15)
                tbox.setText("Turning")
                tbox = self.debug.addText("line2", 0, 30, 100, 15)
                tbox.setText("")
                tbox = self.debug.addText("line3", 0, 45, 100, 15)
                tbox.setText("")
            # self.tauntbox = Gooey.Plain("taunt", 10, 175, 640, 175)
            # tbox = self.tauntbox.addText("taunt1", 10, 0, 640, 15)
            # tbox.setText("")
            
            self.RegisterSmartZone(self.flipZone, 1)            
            self.RegisterSmartZone(self.pushZone, 2)
            self.RegisterSmartZone(self.flameZone, 3)
            
        return AI.SuperAI.Activate(self, active)

    def Tick(self):  
        if self.reloadTime < 0:
            self.reloadTime = 0
    
        if not self.sriComp and not self.flipComp:
            self.bInvertible = True 
            
        if self.flame == 1:
            self.Input(self.FlameIn, 0, 1)
        
        if self.push == 1 and not self.bImmobile:
            self.Input(self.PushIn, 0, 100)
        else:
            self.Input(self.PushIn, 0, 0)
            
        if self.reloadTime > 0: 
            self.reloadTime -= 1
            
    
        self.Input(self.FlipIn, 0, 0)
        
        if self.sriComp:
                self.Input("Srimech", 0, 0)         
        
        # fire srimech if we're stuck on our rear
        
        if list(plus.getDirection(self.GetID(),0))[1]>0.9:
            if self.StuckTimer == 16:
                self.StuckTimer = 8
                if self.sriComp:
                    self.Input("Srimech", 0, 1)
                elif self.flipComp:
                    self.Input(self.FlipIn, 0, 1) 
            else: 
                self.StuckTimer +=1
        else:
            self.StuckTimer = 0

        self.CheckSensors()
                
        if self.flipComp and not self.reloadTime and self.flipLimit: 
            if self.botinzone == 1:            
                self.Input(self.FlipIn, 0, 1)               
                self.reloadTime = self.reloadTime + self.reloadDelay
                self.reloadDelay = self.reloadDelay + self.flipDelay
                self.flipLimit = self.flipLimit - 1                  
            
        return AI.SuperAI.Tick(self)
        
    def CanDriveUpsideDown(self, bot):
        MOVE_THRESHOLD = self.move_thresh
        
        if not self.move_time:
            return True
            
        if not MOVE_THRESHOLD:
            return True
        
        if bot in self.upTrack:
            
            t = self.upTrack[bot]
            # self.tauntbox.get("taunt1").setText(str(t.contact))
            
            # Check if we've made contact with the enemy
            if self.botinzone == 1 and t.contact == 0:
                t.contact = 1

            # check to see if he's moved recently
            position = plus.getLocation(bot)
            time = plus.getTimeElapsed()
            
            if time - t.last_time > self.move_time:
                # this record is too old to be reliable
                t.last_time = time
                t.last_position = position
                #return False
            v0 = vector3(t.last_position)
            v1 = vector3(position)
            # Don't assume the enemy is immobile if we haven't made contact
            if (v1-v0).length() > MOVE_THRESHOLD or t.contact == 0:
                # Reality check: Is the enemy actually still mobile (after a cooldown period to let them settle) - if so, reset contact
                if t.contact == 2 and time - t.stoptime > self.cooldown:
                    t.contact = 0
                    return True
                else:
                    return True
            # Check if we've made contact and the enemy isn't moving
            if (v1-v0).length() <= MOVE_THRESHOLD and t.contact > 0:
                if t.contact == 1:
                    t.contact = 2
                    t.stoptime = plus.getTimeElapsed()
                return False
        else:
            t = UpsideDownTracker()
            t.last_position = plus.getLocation(bot)
            t.last_time = plus.getTimeElapsed()
            t.contact = 0
            t.stoptime = plus.getTimeElapsed()
            self.upTrack[bot] = t
            return False
    
            
    def GetNearestEnemy(self):
        closest = None
        min_dist = 9999
        min_weighted_dist = 9999

        enemies = self.GetEnemies()

        # ignore bots that aren't moving
        # lol this target human players first code was actually useful.
        for enemy in enemies:
            dist = self.GetDistanceToID(enemy)
            weighted_dist = dist
            if not self.CanDriveUpsideDown(enemy):
                weighted_dist = weighted_dist + self.flippedBias
            else:
                weighted_dist = dist
            if weighted_dist < min_weighted_dist:
                closest = enemy
                min_weighted_dist = weighted_dist
                min_dist = dist
         
        

        return closest, min_dist
            
    def InvertHandler(self):       
        while 1:
            move = 100 * self.SrimechMove
                
            for i in range(0, self.sritime):
                if i == 0:
                    if self.sriComp:
                        self.Input("Srimech", 0, 1)

                    elif self.flipComp:
                        self.Input(self.FlipIn, 0, 1)

                if self.SrimechMove:
                    self.Input("Forward", 0, move)
                yield 0


                
    def LostComponent(self, id):  
    
        if id in self.flipComp: 
            self.flipComp.remove(id)
            
        if id in self.sriComp: 
            self.sriComp.remove(id)                    
            
        return AI.SuperAI.LostComponent(self, id)
        
    def StuckHandler(self):
        "This default generator is called when the bot is almost immobile."
        while 1:

            # back up for 2 seconds (will stop once we're not immobile)
            for i in range(0, 16):
                pos = vector3(self.GetLocation())
                dir = vector3(self.GetDirection())
                self.DriveToLocation((pos - dir * 3).asTuple(), True)                
                yield 0

            # go forward for 2 seconds
            for i in range(0, 16):
                pos = vector3(self.GetLocation())
                dir = vector3(self.GetDirection())
                self.DriveToLocation((pos + dir * 3).asTuple())
                if i > 8:
                    self.Input("Srimech", 0, 1)
                    self.Input(self.FlipIn, 0, 1)                
                yield 0
        
    def DebugString(self, id, string):
        if self.debug:
            if id == 0: self.debug.get("line0").setText(string)
            elif id == 1: self.debug.get("line1").setText(string)
            elif id == 2: self.debug.get("line2").setText(string)
            elif id == 3: self.debug.get("line3").setText(string)
            
    def SmartZoneEvent(self, direction, id, robot, chassis):
        if id == 1:
            if robot > 0:
                if direction == 1:
                    self.botinzone = 1
                elif direction == -1:
                    self.botinzone = 0
        
        if id == 2:
            if robot > 0:
                if direction == 1:
                    self.push = 1
                elif direction == -1:
                    self.push = 0
                    
        if id == 3:
            if robot > 0:
                if direction == 1:
                    self.flame = 1
                elif direction == -1:
                    self.flame = 0

        return True
    
AI.register(ENTITY)